"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebCodeLoader = void 0;
const container_definitions_1 = require("@fluidframework/container-definitions");
const scriptManager_1 = require("./scriptManager");
class WebCodeLoader {
    constructor(codeResolver, allowList) {
        this.codeResolver = codeResolver;
        this.allowList = allowList;
        this.loadedModules = new Map();
        this.scriptManager = new scriptManager_1.ScriptManager();
    }
    async seedModule(source, maybeFluidModule) {
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (resolved.resolvedPackageCacheId !== undefined
            && this.loadedModules.has(resolved.resolvedPackageCacheId)) {
            return;
        }
        const fluidModule = maybeFluidModule !== null && maybeFluidModule !== void 0 ? maybeFluidModule : this.load(source);
        if (resolved.resolvedPackageCacheId !== undefined) {
            this.loadedModules.set(resolved.resolvedPackageCacheId, fluidModule);
        }
    }
    async preCache(source) {
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (container_definitions_1.isFluidBrowserPackage(resolved.resolvedPackage)) {
            return this.scriptManager.preCacheFiles(resolved.resolvedPackage.fluid.browser);
        }
    }
    /**
     * @param source - Details of where to find chaincode
     */
    async load(source) {
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (resolved.resolvedPackageCacheId !== undefined) {
            const maybePkg = this.loadedModules.get(resolved.resolvedPackageCacheId);
            if (maybePkg !== undefined) {
                return maybePkg;
            }
        }
        const fluidModuleP = this.loadModuleFromResolvedCodeDetails(resolved);
        if (resolved.resolvedPackageCacheId !== undefined) {
            this.loadedModules.set(resolved.resolvedPackageCacheId, fluidModuleP);
        }
        return fluidModuleP;
    }
    async loadModuleFromResolvedCodeDetails(resolved) {
        if (this.allowList !== undefined && !(await this.allowList.testSource(resolved))) {
            throw new Error("Attempted to load invalid code package url");
        }
        if (!container_definitions_1.isFluidBrowserPackage(resolved.resolvedPackage)) {
            throw new Error(`Package ${resolved.resolvedPackage.name} not a Fluid module.`);
        }
        const loadedScriptsP = this.scriptManager.loadLibrary(resolved.resolvedPackage.fluid.browser.umd);
        let fluidModule;
        for (const script of await loadedScriptsP) {
            const maybeFluidModule = script.entryPoint;
            if (maybeFluidModule.fluidExport !== undefined) {
                if (fluidModule !== undefined) {
                    throw new Error("Multiple Fluid modules loaded");
                }
                fluidModule = maybeFluidModule;
            }
        }
        if ((fluidModule === null || fluidModule === void 0 ? void 0 : fluidModule.fluidExport) === undefined) {
            throw new Error("Entry point of loaded code package not a Fluid module");
        }
        return { module: fluidModule, details: resolved };
    }
}
exports.WebCodeLoader = WebCodeLoader;
//# sourceMappingURL=webLoader.js.map